<?php

if ( ! class_exists( 'SLP_Option_Handler' ) ):

	require_once( SLPLUS_PLUGINDIR . 'include/base_class.object.php' );

	/**
	 * Class SLP_Option_Manager
	 *
	 * Methods to help manage the SLP options.
	 *
	 * @package   StoreLocatorPlus\Options\Manager
	 * @author    Lance Cleveland <lance@charlestonsw.com>
	 * @copyright 2016 Charleston Software Associates, LLC
	 *
	 * @since     4.3.20
	 *
	 * @property    string[] $text_options       An array of smart option slugs that are text options.
	 * @property    array    $change_callbacks   Stack (array) of callbacks in array ( _func_ , _params_ ) format.
	 */
	class SLP_Option_Handler extends SLPlus_BaseClass_Object {
		public $text_options;
		protected $change_callbacks = array();

		/**
		 * Get the smart option value, from legacy $slplus->options/options_nojs if necessary.
		 *
		 * @param string $property
		 *
		 * @return mixed     null if not set or the value
		 */
		public function get( $property ) {
			if ( property_exists( $this, $property ) ) {

				// Set value of smart option from SLP options or options_nojs
				//
				if ( ! isset( $this->$property->value ) ) {
					if ( $this->$property->use_in_javascript ) {
						$this->$property->value = array_key_exists( $property, $this->slplus->options ) ? $this->slplus->options[ $property ] : null;
					} else {
						$this->$property->value = array_key_exists( $property, $this->slplus->options_nojs ) ? $this->slplus->options_nojs[ $property ] : null;
					}

					// Default If Empty
					//
					if ( empty( $this->$property->value ) ) {
						$this->$property->value = $this->$property->default_if_empty;
					}
				}

				/**
				 * FILTER: slp_get_option_<property>
				 *
				 * <property> can be a slplus->options or slplus->options_nojs array key
				 */
				return apply_filters( 'slp_get_option_' . $property, $this->$property->value );
			}

			return null;
		}

		/**
		 * Set the smart option value and the legacy options/options_nojs
		 *
		 * @param $property
		 * @param $value
		 */
		public function set( $property, $value ) {
			if ( property_exists( $this, $property ) ) {
				$this->$property->value = $value;

				if ( $this->$property->use_in_javascript ) {
					if ( method_exists( $this, 'set_valid_options ' ) ) {
						$this->set_valid_options( $value, $property );
					}
				} else {
					if ( method_exists( $this, 'set_valid_options_nojs ' ) ) {
						$this->set_valid_options_nojs( $value, $property );
					}
				}
			}
		}

		/**
		 * Create smart option objects and set to default_if_empty values.
		 *
		 * @param array $smart_option_params array of options arrays = array ( 'slug' , ... )
		 * @param array $options             the options array for this plugin
		 * @param array $options_nojs        the nojs options for this plugin
		 */
		public function create_smart_options( $smart_option_params, &$options, &$options_nojs ) {
			require_once( 'SLP_Option.php' );

			foreach ( $smart_option_params as $option_params ) {
				$property = $option_params['slug'];
				if ( property_exists( $this, $property ) ) {
					$this->$property = new SLP_Option( $option_params );

					if ( $this->$property->use_in_javascript ) {
						$options[ $property ] = $this->$property->default_if_empty;
					} else {
						$options_nojs[ $property ] = $this->$property->default_if_empty;
					}

					// Text Option
					if ( $this->$property->is_text ) {
						$this->text_options[] = $property;
					}
				}

			}
		}

		/**
		 * Return the property formatted option name.
		 *
		 * @param $property
		 *
		 * @return string
		 */
		public function get_option_name( $property ) {
			if ( property_exists( $this, $property ) ) {
				$base_setting = $this->$property->use_in_javascript ? 'options' : 'options_nojs';

				return "${base_setting}[{$property}]";
			}

			return $property;
		}

		/**
		 * Return a list of option slugs that are text options.
		 *
		 * @return string[]
		 */
		public function get_text_options() {
			if ( ! isset( $this->text_options ) ) {
				foreach ( $this->smart_options as $slug => $option ) {
					if ( $option->is_text ) {
						$this->text_options[] = $option->slug;
					}
				}
			}

			return $this->text_options;
		}

		/**
		 * Set an option in an array only if the key already exists, for empty values set to default.
		 *
		 * @param mixed  $val - the value of a form var
		 * @param string $key - the key for that form var
		 * @param        $option_array
		 * @param        $default_array
		 */
		public function set_valid_option( $val, $key, &$option_array, $default_array ) {
			if ( array_key_exists( $key, $option_array ) ) {

				// Value specified
				//
				if ( is_numeric( $val ) || is_bool( $val ) || ! empty( $val ) ) {
					$option_array[ $key ] = stripslashes_from_strings_only( $val );

					// No value or empty - set to default from smart options first then default array
					//
				} else {
					if ( property_exists( $this, $key ) ) {
						$option_array[ $key ] = $this->$key->default_if_empty;
					} else {
						$option_array[ $key ] = $default_array[ $key ];
					}
				}

				// Set smart option value to our options array value
				//
				if ( property_exists( $this, $key ) ) {

					// Value Changed
					//
					if ( $this->$key->value !== $option_array[ $key ] ) {

						if ( isset( $this->$key->value ) ) {
							$original_value = $this->$key->value;
						} else {
							$original_value = null;
						}

						$this->$key->value = $option_array[ $key ];

						if ( ! is_null( $original_value ) ) {
							// Callback on change
							//
							if ( ! is_null( $this->$key->call_when_changed ) ) {
								$this->change_callbacks[] = array( $this->$key->call_when_changed , array( $key, $original_value, $this->$key->value ) );
							}
						}
					}
				}
			}
		}

	}

endif;