<?php

if ( ! class_exists( 'SLP_Options' ) ):
	require_once( 'SLP_Option_Handler.php' );

	/**
	 * Class SLP_Options
	 *
	 * The options management for the base plugin (replaces SLPlus->options / SLPlus->options_nojs)
	 *
	 * @package   StoreLocatorPlus\Options
	 * @author    Lance Cleveland <lance@charlestonsw.com>
	 * @copyright 2016 Charleston Software Associates, LLC
	 *
	 * @since     4.6
	 *
	 * @property SLP_Option $distance_unit
	 * @property SLP_Option $google_server_key
	 * @property SLP_Option $initial_radius
	 * @property SLP_Option $invalid_query_message
	 * @property SLP_Option $instructions
	 * @property SLP_Option $label_directions
	 * @property SLP_Option $label_email
	 * @property SLP_Option $label_fax
	 * @property SLP_Option $label_hours
	 * @property SLP_Option $label_image
	 * @property SLP_Option $label_phone
	 * @property SLP_Option $label_radius
	 * @property SLP_Option $label_search
	 * @property SLP_Option $label_website
	 * @property SLP_Option $map_center
	 * @property SLP_Option $map_center_lat
	 * @property SLP_Option $map_center_lng
	 * @property SLP_Option $message_no_results
	 * @property SLP_Option $message_no_api_key
	 *
	 * @property-read  boolean $initial_distance_already_calculated  - only do this once per change set.
	 *
	 */
	class SLP_Options extends SLP_Option_Handler {
		public $distance_unit;
		public $google_server_key;
		public $initial_radius;
		public $invalid_query_message;
		public $instructions;
		public $label_directions;
		public $label_email;
		public $label_fax;
		public $label_hours;
		public $label_image;
		public $label_phone;
		public $label_radius;
		public $label_search;
		public $label_website;
		public $map_center;
		public $map_center_lat;
		public $map_center_lng;
		public $message_no_results;
		public $message_no_api_key;

		private $initial_distance_already_calculated = false;

		/**
		 * Things we do at the start.
		 */
		public function initialize() {
			$smart_options = array();

			$smart_options[] = array(
				'slug'              => 'distance_unit',
				'default_if_empty'  => 'miles',
				'call_when_changed' => array( $this , 'recalculate_initial_distance' ),
				'use_in_javascript' => true,
			);
			$smart_options[] = array(
				'slug' => 'google_server_key',
			);
			$smart_options[] = array(
				'slug'              => 'initial_radius',
				'use_in_javascript' => true,
			);
			$smart_options[] = array(
				'slug'    => 'instructions',
				'is_text' => true,
			);
			$smart_options[] = array(
				'slug'    => 'invalid_query_message',
				'is_text' => true,
			);
			$smart_options[] = array(
				'slug'              => 'label_directions',
				'is_text'           => true,
				'use_in_javascript' => true,
			);
			$smart_options[] = array(
				'slug'              => 'label_email',
				'is_text'           => true,
				'use_in_javascript' => true,
			);
			$smart_options[] = array(
				'slug'              => 'label_fax',
				'is_text'           => true,
				'use_in_javascript' => true,
			);
			$smart_options[] = array(
				'slug'    => 'label_hours',
				'is_text' => true,
			);
			$smart_options[] = array(
				'slug'    => 'label_image',
				'is_text' => true,
			);
			$smart_options[] = array(
				'slug'              => 'label_phone',
				'is_text'           => true,
				'use_in_javascript' => true,
			);
			$smart_options[] = array(
				'slug'    => 'label_radius',
				'is_text' => true,
			);
			$smart_options[] = array(
				'slug'    => 'label_search',
				'is_text' => true,
			);
			$smart_options[] = array(
				'slug'              => 'label_website',
				'is_text'           => true,
				'use_in_javascript' => true,
			);
			$smart_options[] = array(
				'slug'              => 'map_center',
				'use_in_javascript' => true,
				'call_when_changed' => array( $this , 'recalculate_initial_distance' ),
			);
			$smart_options[] = array(
				'slug'              => 'map_center_lat',
				'use_in_javascript' => true,
				'call_when_changed' => array( $this , 'recalculate_initial_distance' ),
			);
			$smart_options[] = array(
				'slug'              => 'map_center_lng',
				'use_in_javascript' => true,
				'call_when_changed' => array( $this , 'recalculate_initial_distance' ),
			);
			$smart_options[] = array(
				'slug'              => 'message_no_results',
				'is_text'           => true,
				'use_in_javascript' => true,
			);
			$smart_options[] = array(
				'slug'              => 'message_no_api_key',
				'is_text'           => true,
				'use_in_javascript' => true,
			);

			$this->create_smart_options( $smart_options, $this->slplus->options, $this->slplus->options_nojs );
		}

		/**
		 * Execute the stack of change callbacks.
		 *
		 * Use this to run callbacks after all options have been updated.
		 */
		public function execute_change_callbacks() {
			if ( ! empty( $this->change_callbacks ) ) {
				foreach ( $this->change_callbacks as $callback_info ) {
					call_user_func_array( $callback_info[0] , $callback_info[1] );
					$this->change_callbacks = array();
				}
			}
		}

		/**
		 * Get string defaults.
		 *
		 * @param string $key key name for string to translate
		 *
		 * @return string
		 */
		private function get_string_default( $key ) {
			$this->slplus->create_object_text_manager();
			$text_to_return = $this->slplus->text_manager->get_option_default( $key );
			if ( empty( $text_to_return ) ) {
				$text_to_return = apply_filters( 'slp_string_default', '', $key );
			}

			return $text_to_return;
		}

		/**
		 * Things we do once after the plugins are loaded.
		 */
		public function initialize_after_plugins_loaded() {
			$this->set_text_string_defaults();
			$this->slp_specific_setup();
		}

		/**
		 * Recalculate the initial distance for a location from the map center.
		 *
		 * Called if 'distance_unit' changes.
		 */
		public function recalculate_initial_distance( $key, $old_val, $new_val ) {
			if ( ! $this->initial_distance_already_calculated ) {
				$this->slplus->create_object_location_manager();
				$this->slplus->location_manager->recalculate_initial_distance();
				$this->initial_distance_already_calculated = true;
			}
		}

		/**
		 * Set text string defaults.
		 */
		private function set_text_string_defaults() {
			foreach ( $this->get_text_options() as $key ) {

				if ( array_key_exists( $key, $this->slplus->options ) ) {
					$this->slplus->options[ $key ] = $this->get_string_default( $key );

				} elseif ( array_key_exists( $key, $this->slplus->options_nojs ) ) {
					$this->slplus->options_nojs[ $key ] = $this->get_string_default( $key );

				}
			}
		}

		/**
		 * Initialize the options properties from the WordPress database.
		 */
		private function slp_specific_setup() {

			// Set options defaults to values set in property definition above.
			//
			$this->slplus->options['map_home_icon'] = SLPLUS_ICONURL . 'box_yellow_home.png';
			$this->slplus->options['map_end_icon']  = SLPLUS_ICONURL . 'bulb_azure.png';

			// Serialized Options from DB for JS parameters
			//
			$this->slplus->options_default = $this->slplus->options;
			$dbOptions                     = $this->slplus->option_manager->get_wp_option( 'js' );
			if ( is_array( $dbOptions ) ) {
				array_walk( $dbOptions, array( $this, 'set_valid_options' ) );
			}

			// Map Center Fallback
			//
			$this->slplus->recenter_map();

			// Load serialized options for noJS parameters
			//
			$this->slplus->options_nojs_default = $this->slplus->options_nojs;
			$dbOptions                          = $this->slplus->option_manager->get_wp_option( 'nojs' );
			if ( is_array( $dbOptions ) ) {
				array_walk( $dbOptions, array( $this, 'set_valid_options_nojs' ) );
			}
			$this->slplus->javascript_is_forced = $this->slplus->is_CheckTrue( $this->slplus->options_nojs['force_load_js'] );

			$this->set_to_default_if_empty();
		}

		/**
		 * Set valid slplus->options and copy to smart_options
		 *
		 * @param $val
		 * @param $key
		 */
		public function set_valid_options( $val , $key ) {
			$this->set_valid_option( $val, $key, $this->slplus->options, $this->slplus->options_default );
		}

		/**
		 * Set valid slplus->options_nojs and copy to smart_options
		 *
		 * @param $val
		 * @param $key
		 */
		public function set_valid_options_nojs( $val , $key ) {
			$this->set_valid_option( $val, $key, $this->slplus->options_nojs , $this->slplus->options_nojs_default );
		}

		/**
		 * If option is empty then set it to the default.
		 *
		 * Defaults are in the slplus->defaults property.
		 *
		 * options              defaults
		 * bubblelayout         bubblelayout
		 *                      layout
		 *                      maplayout
		 * results_layout       resultslayout
		 *                      searchlayout
		 */
		private function set_to_default_if_empty() {
			foreach ( $this->slplus->defaults as $name => $value ) {
				if ( empty( $this->slplus->options[ $name ] ) && ( ! empty( $value ) ) ) {
					$this->slplus->options[ $name ] = $value;
				}
			}
		}
	}

endif;